// $Header: /home/cvsroot/yaz++/zoom/master-header,v 1.4 2002/10/10 22:47:51 mike Exp $
//
// ZOOM C++ Binding.
// The ZOOM homepage is at http://zoom.z3950.org/
//
// Derived from version 1.0g at
//	http://zoom.z3950.org/bind/cplusplus/zoom-1.0g.hh

#include <stddef.h>		// for size_t

namespace ZOOM {
  // Forward declarations for type names.
  class query;
  class resultSet;
  class record;

  const char *option (const char *key);
  const char *option (const char *key, const char *val);
  int errcode ();
  char *errmsg ();
  char *addinfo ();

  class connection {
  public:
    connection (const char *hostname, int portnum);
    ~connection ();
    const char *option (const char *key) const;
    const char *option (const char *key, const char *val);
  };

  class query {
      // pure virtual class: derive concrete subclasses from it.
  public:
    virtual ~query ();
  };

  class prefixQuery : public query {
  public:
    prefixQuery (const char *pqn);
    ~prefixQuery ();
  };

  class CCLQuery : public query {
  public:
    CCLQuery (const char *ccl, void *qualset);
    ~CCLQuery ();
  };

  class resultSet {
  public:
    resultSet (connection &c, const query &q);
    ~resultSet ();
    const char *option (const char *key) const;
    const char *option (const char *key, const char *val);
    size_t size () const;
    const record *getRecord (size_t i) const;
  };

  class record {
  public:
    ~record ();
    enum syntax {
      UNKNOWN, GRS1, SUTRS, USMARC, UKMARC, XML
    };
    record *clone () const;
    syntax recsyn () const;
    const char *render () const;
    const char *rawdata () const;
  };

  class exception {
  public:
    exception (int code);
    int errcode () const;
    const char *errmsg () const;
  };

  class systemException: public exception {
  public:
    systemException ();
    int errcode () const;
    const char *errmsg () const;
  };

  class bib1Exception: public exception {
  public:
    bib1Exception (int errcode, const char *addinfo);
    int errcode () const;
    const char *errmsg () const;
    const char *addinfo () const;
  };

  class queryException: public exception {
  public:
    static const int PREFIX = 1;
    static const int CCL = 2;
    queryException (int qtype, const char *source);
    int errcode () const;
    const char *errmsg () const;
    const char *addinfo () const;
  };
}
